from typing import List

from mag_tools.enums.symbol import Symbol

from mag_db.bean.column import Column
from mag_db.bean.table import Table
from mag_db.builder.base_sql import BaseSql
from mag_db.enums.engine_type import EngineType
from mag_db.enums.operator_type import OperatorType


class Create(BaseSql):
    def __init__(self, table: Table):
        super().__init__('')

        table.check()

        # 添加"Create TABLE 表名1，表名2... ("
        self.append_operator(OperatorType.CREATE_TABLE).append_tables([table]).append_symbol(Symbol.OPEN_PAREN)

        # 添加列名清单
        self.append_whole_columns(table.columns)

        # 添加唯一约束
        self.appendUnique(table.uniques)

        # 添加主键
        self.appendPrimaryKey(table.primary_keys)

        self.append_symbol(Symbol.CLOSE_PAREN)

        self.appendParams(table.comment, table.auto_increment)

        print(f"创建表: {self}")

    def appendPrimaryKey(self, primary_keys: str):
        if primary_keys:
            self.append_operator(OperatorType.PRIMARY_KEY) \
                .append_string_with_symbol(primary_keys, Symbol.OPEN_PAREN)
        else:
            # 移除最后的逗号
            self.remove_last_char().removeLastChar()

    def appendUnique(self, columns: List[Column]):
        for column in columns:
            self.append_operator(OperatorType.UNIQUE) \
                .appendString_with_symbol(",".join(column.uniques), Symbol.OPEN_PAREN) \
                .append_symbol(Symbol.COMMA)

    def appendParams(self, comment: str, auto_increment: int):
        self.append_operator(OperatorType.ENGINE).append_symbol(Symbol.EQUAL).append_string(EngineType.INNO_DB.value)
        if auto_increment > 0:
            self.append_string(OperatorType.AUTO_INCREMENT.name).append_symbol(Symbol.EQUAL).append_long(auto_increment)
        self.append_operator(OperatorType.DEFAULT_CHARSET).append_symbol(Symbol.EQUAL).append_string("utf8") \
            .append_operator(OperatorType.COMMENT).append_symbol(Symbol.EQUAL).append_comment(comment)

if __name__ == '__main__':
    # 示例代码
    _columns = [Column(name="id"), Column(name="name")]
    _table = Table(name="example_table", _columns=_columns, primary_keys="id", comment="示例表", auto_increment=1)
    _create_sql = Create(_table)
    print(_create_sql)



import sqlparse
from sqlparse.sql import Statement

from mag_db.bean.table_info import TableInfo
from mag_db.parser.base_parser import BaseParser


class DeleteParser(BaseParser):
    def _parse(self, stmt: Statement) -> TableInfo:
        # 提取表名及别名
        tables = super()._find_table(stmt, {'FROM'})

        # 提取条件部分
        where = super()._find_where(stmt)

        return TableInfo(tables, where=where)

if __name__ == "__main__":
    # 示例用法
    _sql = "DELETE FROM table_name1 AS t WHERE id = 1;"

    _stmt = sqlparse.parse(_sql)[0]

    parser = DeleteParser(_stmt)
    if parser.table_info:
        print(parser.table_info)



import unittest

from mag_tools.enums.base_enum import BaseEnum

from mag_db.handler.int_type_handler import IntTypeHandler
from mag_db.handler.string_type_handler import StringTypeHandler
from mag_db.utils.dao_utils import DaoUtils


class TestIntEnum(BaseEnum):
    VALUE1 = (1, '1吧')
    VALUE2 = (2, '2吧')

class TestStrEnum(BaseEnum):
    VALUE1 = ('1', '1吧')
    VALUE2 = ('2', '2吧')


class TestDaoUtils(unittest.TestCase):
    def test_get_type_handler_with_int_enum(self):
        handler = DaoUtils.get_type_handler(TestIntEnum)
        self.assertIsInstance(handler, IntTypeHandler)

    def test_get_type_handler_with_str_enum(self):
        handler = DaoUtils.get_type_handler(TestStrEnum)
        self.assertIsInstance(handler, StringTypeHandler)

    def test_get_type_handler_with_int(self):
        handler = DaoUtils.get_type_handler(int)
        self.assertIsInstance(handler, IntTypeHandler)

    def test_get_type_handler_with_str(self):
        handler = DaoUtils.get_type_handler(str)
        self.assertIsInstance(handler, StringTypeHandler)

if __name__ == '__main__':
    unittest.main()




from mag_tools.enums.base_enum import BaseEnum
from mag_db.enums.sql_type import SqlType


class SqlServerConvertType(BaseEnum):
    """
    SqlServer的列数据类型-SqlType映射的枚举类

    @description: 建立列类型、对应的类名的映射关系
    @version: v1.2
    @date: 2017/7/22
    """
    # 字符与文本
    CHAR = ("CHAR", SqlType.CHAR, "字符串")
    VARCHAR = ("VARCHAR", SqlType.VARCHAR, "可变字符串")
    NVARCHAR = ("NVARCHAR", SqlType.VARCHAR, "可变字符串")
    NCHAR = ("NCHAR", SqlType.CHAR, "字符串")
    TEXT = ("TEXT", SqlType.TEXT, "文本")
    NTEXT = ("NTEXT", SqlType.TEXT, "文本")
    LONGTEXT = ("LONGTEXT", SqlType.TEXT, "长文本")
    MEDIUMTEXT = ("MEDIUMTEXT", SqlType.TEXT, "中长文本")
    # 其它
    BIT = ("BIT", SqlType.BIT, "位")
    BLOB = ("BLOB", SqlType.BLOB, "BLOB")
    BINARY = ("BINARY", SqlType.BINARY, "二进制")
    VARBINARY = ("VARBINARY", SqlType.VARBINARY, "可变二进制")
    BOOLEAN = ("BOOLEAN", SqlType.BOOLEAN, "布尔值")
    IMAGE = ("IMAGE", SqlType.BLOB, "图像")
    # 日期和时间
    DATE = ("DATE", SqlType.DATE, "日期")
    TIME = ("TIME", SqlType.TIME, "时间")
    DATETIME = ("DATETIME", SqlType.DATETIME, "日期时间")
    SMALLDATETIME = ("SMALLDATETIME", SqlType.DATETIME, "日期时间")
    DATETIME2 = ("DATETIME2", SqlType.DATETIME, "日期时间")
    DATETIMEOFFSET = ("DATETIMEOFFSET", SqlType.DATETIME, "日期时间")
    TIMESTAMP = ("TIMESTAMP", SqlType.VARCHAR, "时间戳")
    # 数值
    TINYINT = ("TINYINT", SqlType.TINYINT, "小整数")
    SMALLINT = ("SMALLINT", SqlType.SMALLINT, "短整数")
    BIGINT = ("BIGINT", SqlType.BIGINT, "长整数")
    INT = ("INT", SqlType.INT, "整数")
    REAL = ("REAL", SqlType.REAL, "浮点数")
    FLOAT = ("FLOAT", SqlType.FLOAT, "浮点数")
    DOUBLE = ("DOUBLE", SqlType.DOUBLE, "浮点数")
    DECIMAL = ("DECIMAL", SqlType.DECIMAL, "大数")
    NUMERIC = ("NUMERIC", SqlType.DECIMAL, "大数")
    MONEY = ("MONEY", SqlType.DECIMAL, "对象")

    def __init__(self, code: str, mysql: SqlType, desc: str):
        super().__init__(code, desc)
        self.mysql = mysql


from typing import Optional

from mag_tools.config.sys_config import SysConfig

from mag_db.core.db_session import DBSession
from mag_db.bean.datasource import Datasource


class DatasourceMgr:
    __instance = None

    @classmethod
    def __get_instance(cls):
        if not cls.__instance:
            cls.__instance = cls()
        return cls.__instance

    def __init__(self):
        self.datasources = {}
        for key, value in SysConfig.get_map('datasources').items():
            ds = Datasource.from_config(value)
            ds.name = key.lower()
            self.datasources[ds.name] = ds

    @classmethod
    def get_datasource(cls, datasource_name: str) -> Optional[Datasource]:
        return cls.__get_instance().datasources.get(datasource_name.lower(), None)

    @classmethod
    def create_session(cls, datasource_name: str) -> Optional[DBSession]:
        ds = cls.get_datasource(datasource_name)
        return DBSession(ds) if ds else None


from dataclasses import dataclass, field
from typing import List, Optional

from mag_tools.enums.masking_alg import MaskingAlg

from mag_db.enums.sql_type import SqlType


@dataclass
class Column:
    name: Optional[str] = None  # 列名
    sn: Optional[int] = None
    db_name: Optional[str] = None  # 库名
    table_name: Optional[str] = None  # 表名
    table_sn: Optional[int] = None  # 表序号
    alias: Optional[str] = name  # 别名
    mapping: Optional[str] = None   # 映射到Bean中的字段名
    sql_type: Optional[SqlType] = None  # 列的JDBC类型,比如：CHAR、TINYINT等
    is_unsigned: bool = False  # 是否为无符号数，在TINYINT、INT等有效
    is_zero_filling: bool = False  # 是否用零填充
    data_type: Optional[int] = None  # 数据类型
    is_primary_key: bool = False  # 是否为主键
    uniques: List[str] = field(default_factory=list)  # 是否唯一
    is_can_null: bool = True  # 可否为空
    column_size: Optional[int] = None  # 列大小
    decimal_digits: Optional[int] = None  # 小数位数
    num_prec_radix: Optional[int] = None  # 基数，通常为10或2
    char_octet_length: Optional[int] = None  # 字符类型长度，对char 类型，该长度是列中的最大字节数
    default_value: Optional[str] = None  # 列的缺省值
    ordinal_position: Optional[int] = None  # 列的原始位置
    is_auto_increment: bool = False  # 是否为自增主键
    enum_values: Optional[str] = None  # 枚举的可能值（类型为枚举时有效）
    masking_alg: Optional[MaskingAlg] = None  # 脱敏算法，包括掩码、加密、Hash
    _comment: Optional[str] = None  # 描述

    @classmethod
    def of_whole_name(cls, whole_col_name: Optional[str] = None):
        col = None
        if whole_col_name:
            # 分割表名和列名
            parts = whole_col_name.split(".", 1)
            table_name = parts[0] if len(parts) > 1 else None

            # 分割列名和别名
            parts = parts[-1].lower().split(" as ", 1)
            name = parts[0]
            _alias = parts[1] if len(parts) > 1 else None

            col = cls(name=name, table_name=table_name, alias=_alias, mapping=name)
        return col

    def get_whole_name(self, has_table_name=True, has_alias=True) -> str:
        """
        获取完整格式的列名，格式为：表名.列名 AS 列别名
        :param has_table_name: 是否包含表名
        :param has_alias: 是否包含别名
        :return: 完整格式的列名
        """
        if not self.name:
            return ""

        parts = []

        if has_table_name and self.table_name and not self.is_function:
            parts.append(f"{self.table_name}.")

        if not self.is_function and self.name != "*":
            parts.append(f"{self.name}")
        else:
            parts.append(self.name)

        if has_alias and self.alias:
            parts.append(f" AS {self.alias}")

        return "".join(parts)

    @property
    def short_name(self):
        """
        获取短格式列名，格式为：表名.列名
        :return: 短格式列名
        """
        return f"{self.table_name}.{self.name}" if self.table_name else self.name

    @property
    def is_function(self):
        """
        判定该列名是否为函数，函数名中包含有括号
        :return: 是否为函数
        """
        return self.name and "(" in self.name and ")" in self.name

    @property
    def is_empty(self):
        """
        判定列名是否为空
        :return: 是否为空
        """
        return not self.name

    @property
    def comment(self):
        """
        获取列的描述，去除特殊字符
        :return: 列的描述
        """
        if self._comment:
            self._comment = self._comment.replace("'", "").replace("\"", "")
        return self._comment

    @property
    def is_unique(self):
        """
        判定列是否唯一
        :return: 是否唯一
        """
        return len(self.uniques) > 0

    @classmethod
    def to_field_name(cls, column_name: str) -> str:
        column = cls(name=column_name)
        return column.name


import os
import unittest

from mag_tools.config.sys_config import SysConfig
from mag_tools.utils.common.random_utils import RandomUtils

from mag_db.core.base_dao import BaseDao


class TestBaseDao(unittest.TestCase):
    def setUp(self):
        SysConfig.set_root_dir(os.path.dirname(os.getcwd()))
        self.dao = BaseDao()

    def test_insert_by_sql(self):
        _id = RandomUtils.random_number(5)
        name = RandomUtils.random_string(10)

        insert_sql = f"INSERT INTO test_table (id, name) VALUES ({_id}, '{name}')"
        self.dao.insert_by_sql(insert_sql)

    def test_update_by_sql(self):
        update_sql = f"UPDATE test_table SET name = '{RandomUtils.random_string(10)}' WHERE id = 1"
        self.dao.update_by_sql(update_sql)

    def test_query_by_sql(self):
        query_sql = "SELECT id, name FROM test_table"
        results = self.dao.query_by_sql(query_sql)
        for result in results:
            print(result)

    def test_delete_by_sql(self):
        delete_sql = "DELETE FROM test_table WHERE id = 8"
        self.dao.delete_by_sql(delete_sql)

    def test_total_count(self):
        table_name = "test_table"
        count = self.dao.total_count(table_name)
        print(count)

    def test_test(self):
        result = self.dao.test()
        print(result)

if __name__ == '__main__':
    unittest.main()



from typing import List, Optional

from mag_tools.enums.symbol import Symbol

from mag_db.bean.column import Column
from mag_db.bean.table import Table
from mag_db.builder.base_sql import BaseSql
from mag_db.builder.where import Where
from mag_db.enums.operator_type import OperatorType


class Select(BaseSql):
    """
    SELECT语句操作类

    @description: 用于构建SELECT语句。
    @version: v1.0
    @date: 2019/8/30
    """
    def __init__(self, table_names: List[str], fully_column_names: List[str], condition: Optional[Where]):
        """
        构造方法，适用于多表查询

        :param table_names: 表名与别名的列表，tableName1 alias1,tableName2 alias2
        :param fully_column_names: the column names you want to get，格式：表名.列名 AS 别名，其中表名和别名可为空
        :param condition: the condition
        """
        super().__init__()
        self.tables = Table.to_tables(table_names)
        self.columns = self.to_columns(fully_column_names)
        self.where = condition
        self.reset_sql()


    def set_distinct(self):
        """
        设置为去重查询
        """
        super().replace(OperatorType.SELECT.value(), OperatorType.DISTINCT.value())

    def reset_sql(self):
        self.clear()
        self.append_operator(OperatorType.SELECT)
        self.append_columns(self.columns, self.is_multiple_table()).append_from(self.tables).append_condition(self.where)

    def append_from(self, table_names: List[Table]) -> 'Select':
        self.append_operator(OperatorType.FROM)
        self.append_tables(table_names)
        return self

    def append_columns(self, columns: List[Column], is_multiple_table: bool) -> 'Select':
        """
        添加用逗号分开的列名,(col1, col2, col3)

        :param columns: 列名列表
        :param is_multiple_table: 是否多表
        :return: Select
        """
        if not columns:
            return self
        for i, column in enumerate(columns):
            if i > 0:
                self.append_symbol(Symbol.COMMA)
            self.append_column(column, is_multiple_table, is_multiple_table)
        return self

    def is_multiple_table(self) -> bool:
        return len(self.tables) > 1



from enum import Enum
from typing import Optional, Type, get_origin

from mag_tools.utils.common.enum_utils import EnumUtils

from mag_db.handler.type_handler import TypeHandler
from mag_db.handler.type_handler_factory import TypeHandlerFactory


class DaoUtils:
    @staticmethod
    def get_type_handler(field_type: Type)->Optional[TypeHandler]:
        """
        根据字段类型获取 TypeHandler

        :param field_type: 字段类型
        :return: TypeHandler
        :raises: DaoException
        """
        # 如果为空，表示该字段在 bean 中不存在
        if field_type is None:
            return None

        type_handler = None
        if isinstance(field_type, type) and issubclass(field_type, Enum):
            try:
                enum_value_type = str
                if hasattr(field_type, 'code'):
                    enum_value_type = EnumUtils.get_enum_type(field_type, 'code')

                type_handler = TypeHandlerFactory.get(enum_value_type)
            except AttributeError:
                pass
        else:
            if not isinstance(field_type, type):
                field_type = get_origin(field_type)

            type_handler = TypeHandlerFactory.get(field_type)

        return type_handler

    @staticmethod
    def get_count_sql(sql: str) -> str:
        return f"SELECT COUNT(*) AS COUNT_ FROM ({sql}) AS COUNTTB_"






from mag_db.core.db_session import DBSession
from mag_db.sql.update_executor import UpdateExecutor


class IndexDeleteExecutor(UpdateExecutor):
    def __init__(self, sql: str, session: DBSession):
        super().__init__(sql, session)


import re
from datetime import date, datetime
from decimal import Decimal
from enum import Enum
from typing import Any, List

from mag_db.bean.column import Column
from mag_db.bean.table import Table
from mag_db.data.column_names_mapping import ColumnNamesMapping
from mag_db.enums.db_type import DbType
from mag_db.enums.oracle_convert_type import OracleConvertType
from mag_db.enums.sql_server_convert_type import SqlServerConvertType
from mag_db.enums.sql_type import SqlType


class ColumnUtils:
    MAX_OF_VARCHAR = 8 * 1024
    MAX_OF_TEXT = 64 * 1024 - 1
    MAX_OF_MEDIUM_TEXT = 16 * 1024 * 1024 - 1

    @staticmethod
    def to_field_names(column_names: List[str], column_names_mapping: ColumnNamesMapping) -> List[str]:
        """
        将列名列表变换为bean的字段名的列表
        注：如下列名可自动转换：
        1、列名由多个单词组成时，之间使用"_"连接，全小写字母
        2、列名与bean字段名相同
        3、列名为：表名.列名
        对第三种，去掉表名和点，再转换
        如：数据库字段：user.family_address; Bean字段名：family_address

        :param column_names: 列名列表，格式：列名；表名.列名；表名__列名；表名~列名，列名为首字母小写的驼峰格式或_连接的数据库格式
        :param column_names_mapping: 列名映射表
        :return: bean的字段名列表
        """
        return [ColumnUtils.__to_field_name(column_name, column_names_mapping) for column_name in column_names]

    # @staticmethod
    # def to_columns(column_names: List[str]) -> List[Column]:
    #     return [Column(name[:-1] if name.endswith('_') else name) for name in column_names]

    @staticmethod
    def contains(columns: List[Column], column_name: str) -> bool:
        """
        判定列名列表中是否包含指定列名

        :param columns: 列的列表
        :param column_name: 指定的列名
        :return: 列名列表中是否包含指定列名
        """
        return any(column.name == column_name for column in columns)

    @staticmethod
    def convert_to_mysql(columns: List[Column], db_type: DbType) -> List[Column]:
        def convert_column(column: Column):
            from_type = column.sql_type

            if db_type == DbType.ORACLE:
                column.sql_type = OracleConvertType.of_code(from_type.code).sql_type
            elif db_type == DbType.SQL_SERVER:
                column.sql_type = SqlServerConvertType.of_code(from_type.code).sql_type

            if db_type == DbType.SQL_SERVER:
                if from_type == SqlType.MONEY:
                    column.decimal_digits = 4
                    column.column_size = 19
                elif from_type == SqlType.TIMESTAMP:
                    column.column_size = 64

            if column.sql_type == SqlType.VARCHAR:
                if column.column_size is None:
                    column.column_size = ColumnUtils.MAX_OF_VARCHAR
                elif column.column_size > ColumnUtils.MAX_OF_VARCHAR:
                    sql_type = SqlType.LONGTEXT if column.column_size > ColumnUtils.MAX_OF_MEDIUM_TEXT else SqlType.MEDIUMTEXT
                    column.sql_type = sql_type
                    column.column_size = 0
                    column.char_octet_length = 0
                    if column.default_value is not None:
                        column.default_value = None
            elif column.sql_type == SqlType.BIT:
                default_value = column.default_value
                if default_value is not None:
                    if default_value not in ["b'0'", "b'1'"]:
                        column.sql_type = SqlType.TINYINT
                        column.column_size = 1
                elif column.column_size == 1:
                    column.sql_type = SqlType.TINYINT
                    column.column_size = 1
            elif column.sql_type == SqlType.ENUM:
                column.sql_type = SqlType.VARCHAR
                column.column_size = 128

            return column

        return [convert_column(column) for column in columns]

    @staticmethod
    def to_column(field: Any):
        column_size = None
        decimal_digits = None

        cls = type(field)
        if cls == str:
            column_size = 255
            sql_type = SqlType.of_class(cls, column_size)
        elif cls in [float, int, Decimal]:
            sql_type = SqlType.of_class(cls, column_size)
        elif isinstance(cls, Enum):
            column_size = 32
            sql_type = SqlType.of_class(str, column_size)
        elif cls in [datetime, date]:
            sql_type = SqlType.of_class(cls)
        else:
            sql_type = SqlType.of_class(cls)

        return Column(
            name=field.name[:-1] if field.name.endswith('_') else field.name,
            sql_type=sql_type,
            column_size=column_size,
            decimal_digits=decimal_digits
        )

    @staticmethod
    def convert_column_name(sql: str, tables: List[Table]) -> str:
        for table in tables:
            table_name = table.name
            table_alias = table.alias

            # 列名中含有表别名的，则替换为表名
            if table_alias and '.' in sql:
                pattern = re.compile(rf"(?<!\w){re.escape(table_alias)}\.")
                sql = pattern.sub(f"{table_name}.", sql)

                # 去掉表别名设置，表别名在表名后，空格分隔，后面为空格或逗号
                sql = re.sub(rf"\b{re.escape(table_alias)}\b(?=\s|,)", table_name, sql)
        return sql

    @staticmethod
    def __to_field_name(column_name: str, column_names_mapping: ColumnNamesMapping) -> str:
        field_name = column_names_mapping.get_target_name(column_name)
        if not field_name:
            field_name = Column(column_name).name

        if field_name in {'type', 'str', 'len', 'sum', 'max', 'min', 'input', 'zip', 'range', 'map', 'set', 'dict',
                           'list', 'object', 'super', 'sorted', 'filter', 'print'}:
            field_name += '_'

        return field_name



from dataclasses import dataclass
from typing import List

@dataclass
class ForeignKey:
    """
    描述一个外键约束

    属性:
    column_names (List[str]): 外键列名列表
    referenced_table (str): 引用的表名
    referenced_columns (List[str]): 引用的列名列表
    """
    column_names: List[str]  # 外键列名列表
    referenced_table: str  # 引用的表名
    referenced_columns: List[str]  # 引用的列名列表

    def __str__(self):
        """
        返回外键约束的字符串表示

        返回:
        str: 外键约束的字符串表示
        """
        column_names_str = ', '.join(self.column_names)
        referenced_columns_str = ', '.join(self.referenced_columns)
        return f"FOREIGN KEY ({column_names_str}) REFERENCES {self.referenced_table}({referenced_columns_str})"


from typing import TypeVar, Any

T = TypeVar('T')

class TypeHandler:
    def set_parameter(self, parameter: T) -> Any | None:
        raise NotImplementedError("set_parameter() must be implemented by subclasses")


from typing import List

from mag_tools.enums.symbol import Symbol

from mag_db.bean.column import Column
from mag_db.bean.table import Table
from mag_db.builder.base_sql import BaseSql
from mag_db.builder.where import Where
from mag_db.enums.operator_type import OperatorType


class Update(BaseSql):
    """
    UPDATE语句操作类

    @description: 用于构建UPDATE语句。
    @version: v1.0
    @date: 2019/8/30
    """
    def __init__(self, table_names: List[str], column_names: List[str], where: Where):
        """
        构造方法

        :param table_names: the database table names
        :param column_names: the column names which need to update
        :param where: the string conditions, for example: WHERE condition1 = ? AND condition2 = ?
        """
        super().__init__()

        self.append_operator(OperatorType.UPDATE).append_tables([Table(name) for name in table_names])
        self.append_set_columns(self.to_columns(column_names))
        if where:
            self.append_condition(where)

    def append_set_columns(self, column_names: List[Column]):
        """
        添加列名设置

        :param column_names: the column name which need to update
        """
        self.append_operator(OperatorType.SET)

        for i, column in enumerate(column_names):
            self.append_column(column, True, False)
            self.append_symbol(Symbol.EQUAL)
            self.append_symbol(Symbol.PLACE_HOLDER)
            if i < len(column_names) - 1:
                self.append_symbol(Symbol.COMMA)

        return self


from typing import List

from mag_db.builder.base_sql import BaseSql
from mag_db.bean.column import Column
from mag_db.bean.table import Table
from mag_db.enums.operator_type import OperatorType
from mag_tools.enums.symbol import Symbol

class Alter(BaseSql):
    def __init__(self, table_name: str, columns: List[Column], operator_type: OperatorType):
        super().__init__()

        table = Table(table_name)

        self.append_operator(OperatorType.ALTER_TABLE).append_tables([table])

        if operator_type == OperatorType.ADD:
            for column in columns:
                self.append_operator(OperatorType.ADD).append_whole_column(column)
        elif operator_type == OperatorType.DROP_COLUMN:
            for column in columns:
                self.append_operator(OperatorType.DROP_COLUMN).append_column(column, False, False).append_symbol(Symbol.COMMA)

        self.remove_last_keyword(",")

    @staticmethod
    def add(table_name: str, columns: List[Column]):
        return Alter(table_name, columns, OperatorType.ADD)

    @staticmethod
    def drop(table_name: str, columns: List[Column]):
        return Alter(table_name, columns, OperatorType.DROP_COLUMN)



import sqlparse
from sqlparse.sql import Statement

from mag_db.bean.table_info import TableInfo
from mag_db.parser.base_parser import BaseParser


class DropParser(BaseParser):
    def _parse(self, stmt: Statement) -> TableInfo:
        # 提取表名
        tables = super()._find_table(stmt, {'EXISTS', 'TABLE'})
        return TableInfo(tables)


if __name__ == "__main__":
    # 示例用法
    _sql = "DROP TABLE IF EXISTS table_name1;"

    _stmt = sqlparse.parse(_sql)[0]
    parser = DropParser(_stmt)
    if parser.table_info:
        print(parser.table_info)



from typing import Dict

from mag_tools.log.logger import Logger
from mag_tools.enums.log_type import LogType


class TransactionGroup:
    def __init__(self):
        self.__tx_group = []

    @property
    def current(self):
        if not self.__tx_group:
            return None

        tx = self.__tx_group[-1]
        if tx is None:
            Logger.throw(LogType.DAO, "Cached transaction is null")
        return tx

    def remove(self, tx):
        self.__tx_group.remove(tx)

    def append(self, tx):
        self.__tx_group.append(tx)

class TransactionCache:
    """
    数据库事务缓存管理类
    """
    __transaction_cache : Dict[str, TransactionGroup] = {}

    @classmethod
    def append_tx(cls, ds_name: str, tx):
        tx_group = cls.__get_tx_group_by_name(ds_name)
        tx_group.append(tx)

    @classmethod
    def remove_tx(cls, ds_name: str, tx):
        tx_group = cls.__get_tx_group_by_name(ds_name)
        tx_group.remove(tx)

    @classmethod
    def get_current_tx(cls, ds_name: str):
        tx_group = cls.__get_tx_group_by_name(ds_name)
        return tx_group.current

    @classmethod
    def close(cls):
        cls.__transaction_cache.clear()

    @classmethod
    def __get_tx_group_by_name(cls, ds_name: str) -> TransactionGroup:
        if ds_name not in cls.__transaction_cache:
            cls.__transaction_cache[ds_name] = TransactionGroup()
        return cls.__transaction_cache[ds_name]




import os.path
import random
import unittest
from dataclasses import dataclass
from datetime import datetime
from typing import TypeVar, Optional

from mag_tools.bean.easy_map import EasyMap
from mag_tools.config.sys_config import SysConfig

from mag_db.core.simple_dao import SimpleDao
from mag_db.builder.where import Where

T = TypeVar('T')

@dataclass
class Test:
    name: Optional[str] = None
    age: int = None
    open_date: datetime = None
    id: int = None
    phone_numbers: tuple[str,...] = None

class TestDao(SimpleDao):
    def __init__(self):
        super().__init__(['test_table'], Test, primary_key_column='id', auto_increment=True)


class TestSimpleDao(unittest.TestCase):
    def setUp(self):
        SysConfig.set_root_dir(os.path.dirname(os.getcwd()))

    def test_insert(self):
        age = random.randint(1, 100)
        bean = Test("Alice", age, phone_numbers=('19216808998','19216808999',))
        TestDao().insert(bean)

        result = TestDao().query_map(Where(f"Where age = {age} and name = 'Alice'"))

        print(result)

    def test_insert_map(self):
        age = random.randint(1, 100)
        field = {"age": age, "name": "Alice"}
        id_ = TestDao().insert_map(EasyMap(field))
        result = TestDao().query_map(Where(f"Where age = {age} and name = 'Alice'"))

        self.assertEqual(result.get('id'), id_)

    def test_insert_beans(self):
        age1 = random.randint(1, 100)
        age2 = random.randint(1, 100)
        age3 = random.randint(1, 100)

        beans = [Test("Alice", age1, datetime.now()), Test("Bob", age2), Test("Peter", age3)]
        ids = TestDao().insert_beans(beans)

        where = Where(f"Where age = {age1} and name = 'Alice'")
        id1 = TestDao().query(Test, where).id

        where = Where(f"Where age = {age2} and name = 'Bob'")
        id2 = TestDao().query(Test, where).id

        where = Where(f"Where age = {age3} and name = 'Peter'")
        id3 = TestDao().query(Test, where).id

        self.assertEqual([id1, id2, id3], ids)

    def test_insert_maps(self):
        age1 = random.randint(1, 100)
        age2 = random.randint(1, 100)
        age3 = random.randint(1, 100)

        fields = [{"age": age1, "name": "Alice"}, {"age": age2, "name": "Bob"}, {"age": age3, "name": "Peter"}]
        ids = TestDao().insert_maps([EasyMap(field) for field in fields])

        where = Where(f"Where age = {age1} and name = 'Alice'")
        id1 = TestDao().query(Test, where).id

        where = Where(f"Where age = {age2} and name = 'Bob'")
        id2 = TestDao().query(Test, where).id

        where = Where(f"Where age = {age3} and name = 'Peter'")
        id3 = TestDao().query(Test, where).id

        self.assertEqual([id1, id2, id3], ids)

    def test_delete(self):
        age_1 = random.randint(100, 10000)
        bean = Test("Alice", age_1)
        result = TestDao().insert(bean)
        print(f"插入的ID：{result}")

        where = Where(f"age = {age_1} AND name = 'Alice'")
        result = TestDao().delete(where)
        self.assertEqual(result, 1)

    def test_delete_by_id(self):
        age_1 = random.randint(100, 10000)
        bean = Test("Alice", age_1)
        id_ = TestDao().insert(bean)
        print(f"插入的ID：{id_}")

        result = TestDao().delete_by_id(id_)
        self.assertEqual(result, 1)

    def test_delete_by_ids(self):
        age1 = random.randint(1, 100)
        age2 = random.randint(1, 100)

        fields = [{"age": age1, "name": "Alice"}, {"age": age2, "name": "Bob"}]
        ids = TestDao().insert_maps([EasyMap(field) for field in fields])

        result = TestDao().delete_by_ids(ids)
        self.assertEqual(result, 2)

    def test_clear(self):
        result = TestDao().clear()
        print(result)

    def test_update(self):
        age = random.randint(1, 100)
        bean = Test("Alice", age)
        id_ = TestDao().insert(bean)

        bean.age = 1000
        where = Where(f"id = {id_}")
        result = TestDao().update(bean, where)
        self.assertEqual(result, 1)

    def test_update_by_id(self):
        age = random.randint(1, 1000)
        bean = Test("Alice", age)
        id_ = TestDao().insert(bean)

        bean.age = 1001
        result = TestDao().update_by_id(bean, id_)
        self.assertEqual(result, 1)

    def test_update_by_id_of_bean(self):
        age = random.randint(1, 1000)
        bean = Test("Alice", age)
        id_ = TestDao().insert(bean)

        bean.id = id_
        bean.age = 1005

        result = TestDao().update_by_id_of_bean(bean)
        self.assertEqual(result, 1)

    def test_update_beans(self):
        age1 = random.randint(1, 100)
        age2 = random.randint(1, 100)
        bean1 = Test("Alice", age1, datetime.now())
        bean2 = Test("Bob", age2, datetime.now())
        ids = TestDao().insert_beans([bean1, bean2])
        bean1.id = ids[0]
        bean1.age = 1009
        bean2.id = ids[1]
        bean2.age = 1010

        result = TestDao().update_beans([bean1, bean2])
        self.assertEqual(result, 2)

    def test_update_map(self):
        age1 = random.randint(1, 100)
        bean1 = Test("Alice", age1, datetime.now())
        ids = TestDao().insert_beans([bean1])

        field_map = {"name": "Alice", "age": 1020}
        where = Where(f"id = {ids[0]}")
        result = TestDao().update_map(field_map, where)
        self.assertEqual(result, 1)

    def test_update_map_by_id(self):
        age1 = random.randint(1, 100)
        map1 = EasyMap().add("name", "Alice").add("age", age1).add("open_date", datetime.now())
        ids = TestDao().insert_maps([map1])

        field_map = {"name": "Alice", "age": 1099}
        result = TestDao().update_map_by_id(field_map, ids[0])

        self.assertEqual(result, 1)

    def test_update_map_by_id_of_map(self):
        age1 = random.randint(1, 100)
        map1 = EasyMap().add("name", "Alice").add("age", age1).add("open_date", datetime.now())
        ids = TestDao().insert_maps([map1])

        field_map = {"id": f"{ids[0]}", "name": "Alice", "age": 1199}
        result = TestDao().update_map_by_id_of_map(field_map)
        self.assertEqual(result, 1)

    def test_update_maps(self):
        age1 = random.randint(1, 100)
        age2 = random.randint(1, 100)
        map1 = EasyMap().add("name", "Alice").add("age", age1).add("open_date", datetime.now())
        map2 = EasyMap().add("name", "Bob").add("age", age2).add("open_date", datetime.now())
        ids = TestDao().insert_maps([map1, map2])

        map1 = {"id": ids[0], "age": 1009, "open_date": datetime.now()}
        map2 = {"id": ids[1], "age": 1010, "open_date": datetime.now()}

        result = TestDao().update_maps([map1, map2])
        self.assertEqual(result, 2)

    def test_query(self):
        where = Where().column_value('id', 99723)
        result = TestDao().query(Test, where)
        print(result)

    def test_query_map(self):
        where = Where().column_value('id', 99669)
        result = TestDao().query_map(where)
        print(result)

    def test_query_by_id(self):
        result = TestDao().query_by_id(Test, 99669)
        print(result)

    def test_query_map_by_id(self):
        result = TestDao().query_map_by_id(99669)
        print(result)

    def test_list(self):
        result = TestDao().list(Test, Where("`id` = 99669"))
        for row in result:
            print(row)

    def test_list_maps(self):
        result = TestDao().list_maps(Where("`id` = 99669"))
        for row in result:
            print(row)

    def test_list_all(self):
        result = TestDao().list_all(Test)
        for row in result:
            print(row)

    def test_list_all_maps(self):
        result = TestDao().list_all_maps()
        for row in result:
            print(row)

    def test_get_latest_time(self):
        dt = TestDao().get_latest_time(Where(), 'open_date')
        print(dt)

if __name__ == '__main__':
    unittest.main()



from dataclasses import dataclass, field
from typing import List, Optional

@dataclass
class PrimaryKey:
    """
    描述一个主键约束

    属性:
    column_names (List[str]): 主键列名列表
    key_seq (Optional[List[int]]): 主键列的顺序列表，默认为 None
    pk_name (Optional[str]): 主键的名称，默认为 None
    increment (Optional[int]): 自增列的增量值，默认为 None
    """
    column_names: List[str] = field(default_factory=list) # 主键列名列表
    key_seq: List[int] = field(default_factory=list)  # 主键列的顺序列表，默认为 None
    pk_name: Optional[str] = None  # 主键的名称，默认为 None
    increment: Optional[int] = None  # 自增列的增量值，默认为 None

    @property
    def default_column(self) -> str:
        return self.column_names[0] if self.column_names and len(self.column_names) == 1 else None

    def __str__(self):
        """
        返回主键约束的字符串表示

        返回:
        str: 主键约束的字符串表示
        """
        parts = [f"PRIMARY KEY ({', '.join(self.column_names)})"]
        if self.key_seq and len(self.key_seq) > 0:
            parts.append(f"KEY_SEQ ({', '.join(map(str, self.key_seq))})")
        if self.pk_name:
            parts.append(f"PK_NAME {self.pk_name}")
        if self.increment:
            parts.append(f"INCREMENT {self.increment}")
        return " ".join(parts)


from decimal import Decimal
from types import NoneType
from typing import Type, Dict, get_origin
from datetime import date, time, datetime

from mag_tools.exception.dao_exception import DaoException

from mag_db.handler.none_type_handler import NoneTypeHandler
from mag_db.handler.dict_type_handler import DictTypeHandler
from mag_db.handler.list_type_handler import ListTypeHandler
from mag_db.handler.decimal_type_handler import DecimalTypeHandler
from mag_db.handler.tuple_type_handler import TupleTypeHandler
from mag_db.handler.bool_type_handler import BoolTypeHandler
from mag_db.handler.bytes_type_handler import BytesTypeHandler
from mag_db.handler.date_type_handler import DateTypeHandler
from mag_db.handler.datetime_type_handler import DatetimeTypeHandler
from mag_db.handler.float_type_handler import FloatTypeHandler
from mag_db.handler.int_type_handler import IntTypeHandler
from mag_db.handler.string_type_handler import StringTypeHandler
from mag_db.handler.time_type_handler import TimeTypeHandler
from mag_db.handler.type_handler import TypeHandler


class TypeHandlerFactory:
    __instance = None
    __type_map: Dict[Type, TypeHandler] = {}

    def __init__(self):
        self.__type_map[str] = StringTypeHandler()
        self.__type_map[int] = IntTypeHandler()
        self.__type_map[float] = FloatTypeHandler()
        self.__type_map[Decimal] = DecimalTypeHandler()
        self.__type_map[bool] = BoolTypeHandler()
        self.__type_map[bytes] = BytesTypeHandler()
        self.__type_map[date] = DateTypeHandler()
        self.__type_map[time] = TimeTypeHandler()
        self.__type_map[datetime] = DatetimeTypeHandler()
        self.__type_map[tuple] = TupleTypeHandler()
        self.__type_map[list] = ListTypeHandler()
        self.__type_map[dict] = DictTypeHandler()
        self.__type_map[NoneType] = NoneTypeHandler()

    @classmethod
    def __get_instance(cls):
        if not cls.__instance:
            cls.__instance = cls()
        return cls.__instance

    @classmethod
    def get(cls, cls_type: Type) -> TypeHandler:
        if cls_type is None:
            raise DaoException("javaType is null")

        try:
            cls_type = tuple if get_origin(cls_type) == tuple else cls_type
            return cls.__get_instance().__type_map.get(cls_type)
        except KeyError:
            raise DaoException(f"No TypeHandler found for type: {cls_type}")



from typing import List, Optional

from mag_tools.enums.symbol import Symbol

from mag_db.bean.column import Column
from mag_db.bean.table import Table
from mag_db.enums.operator_type import OperatorType
from mag_db.enums.relation import Relation


class SqlBuilder:
    def __init__(self, sql: Optional[str] = None):
        """
        初始化SqlBuilder类，接受一个可选的SQL字符串。
        :param sql: 可选的SQL字符串
        """
        self.string_builder = [sql] if sql else []

    def append_string(self, string: str):
        """
        添加一个字符串到SQL构建器中，并在前后添加空格。
        :param string: 要添加的字符串
        """
        if string:
            self.remove_last_blank()
            self.string_builder.append(f" {string} ")
        return self

    def append_string_with_symbol(self, string: str, symbol: Symbol):
        """
        添加一个带有符号（如括号、引号）的字符串到SQL构建器中。
        :param string: 要添加的字符串
        :param symbol: 符号，包括单引号、双引号、括号或重音符
        """
        if not string and symbol == Symbol.ACCENT:
            return self

        if string is None:
            string = ""

        self.remove_last_blank()
        close = Symbol.CLOSE_PAREN if symbol == Symbol.OPEN_PAREN else symbol
        self.string_builder.append(f" {symbol}{string}{close} ")

        return self

    def append_long(self, long: int):
        """
        添加一个长整数到SQL构建器中，并在前后添加空格。
        :param long: 要添加的长整数
        """
        if long is not None:
            self.remove_last_blank()
            self.string_builder.append(f" {long} ")
        return self

    def append_long_with_paren(self, long: int):
        """
        添加一个带有括号的长整数到SQL构建器中。
        :param long: 要添加的长整数
        """
        if long is not None:
            self.remove_last_blank()
            self.string_builder.append(f" ({long}) ")
        return self

    def append_operator(self, operator_type: OperatorType):
        """
        添加一个操作符到SQL构建器中，并在前后添加空格。
        :param operator_type: 要添加的操作符
        """
        if OperatorType:
            self.remove_last_blank()
            self.string_builder.append(f" {operator_type.code} ")
        return self

    def append_relation(self, relation: Relation):
        """
        添加一个关系符到SQL构建器中，并在前后添加空格。
        :param relation: 要添加的关系符
        """
        if relation:
            self.remove_last_blank()
            self.string_builder.append(f" {relation.code} ")
        return self

    def append_symbol(self, symbol: Symbol):
        """
        添加一个符号到SQL构建器中，并根据符号类型添加适当的空格。
        :param symbol: 要添加的符号
        """
        if symbol:
            if symbol in [Symbol.OPEN_PAREN, Symbol.PLACE_HOLDER]:
                self.string_builder.append(" ")
            self.string_builder.append(symbol.code)
            if symbol in [Symbol.CLOSE_PAREN, Symbol.COMMA, Symbol.PLACE_HOLDER]:
                self.string_builder.append(" ")
        return self

    def append_table(self, table: Table):
        """
        添加一个表名到SQL构建器中，可选地包含别名。
        :param table: 要添加的表名
        """
        if table.name:
            self.remove_last_blank()
            self.append_string(table.fully_name)
        return self

    def append_column(self, column: Column, has_table_name: bool, has_alias: bool):
        """
        添加一个列名到SQL构建器中，可选地包含表名和别名。
        :param column: 要添加的列名
        :param has_table_name: 是否包含表名
        :param has_alias: 是否包含别名
        """
        if not column.is_empty:
            self.remove_last_blank()
            self.append_string(column.get_whole_name(has_table_name, has_alias))
        return self

    def append_without_paren(self, columns: List[Column], has_table_name: bool):
        """
        添加一个列的列表到SQL构建器中，不包含括号。
        :param columns: 要添加的列的列表
        :param has_table_name: 是否包含表名
        """
        if not columns:
            return self
        for i, column in enumerate(columns):
            if i > 0:
                self.append_symbol(Symbol.COMMA)
            self.append_column(column, has_table_name, False)
        return self

    def append_with_paren(self, columns: List[Column], has_table_name: bool):
        """
        添加一个列的列表到SQL构建器中，并在前后添加括号。
        :param columns: 要添加的列的列表
        :param has_table_name: 是否包含表名
        """
        if not columns:
            return self
        self.append_symbol(Symbol.OPEN_PAREN)
        self.append_without_paren(columns, has_table_name)
        self.append_symbol(Symbol.CLOSE_PAREN)
        return self

    def append_symbol_with_paren(self, symbol: Symbol, param_count: int):
        """
        添加多个符号到SQL构建器中，并用括号包含。
        :param symbol: 要添加的符号
        :param param_count: 符号的个数
        """
        if param_count > 0:
            self.append_symbol(Symbol.OPEN_PAREN)
            for i in range(param_count):
                if i > 0:
                    self.append_symbol(Symbol.COMMA)
                self.append_symbol(symbol)
            self.append_symbol(Symbol.CLOSE_PAREN)
        return self

    def clear(self):
        """
        清空SQL构建器。
        """
        self.string_builder = []

    @property
    def size(self) -> int:
        """
        获取SQL构建器的大小。
        :return: SQL构建器的大小
        """
        return len(self.string_builder)

    @property
    def is_empty(self) -> bool:
        """
        检查SQL构建器是否为空。
        :return: 如果为空则返回True，否则返回False
        """
        return len(self.string_builder) == 0

    def __str__(self) -> str:
        """
        将SQL构建器转换为字符串。
        :return: SQL构建器的字符串表示
        """
        return " ".join(self.string_builder).replace("  ", " ")

    def is_at_end(self, keyword: str) -> bool:
        """
        检查一个关键字是否在SQL构建器的末尾。
        :param keyword: 要检查的关键字
        :return: 如果关键字在末尾则返回True，否则返回False
        """
        return not self.is_empty and self.string_builder[-1].endswith(keyword)

    def remove_last_char(self):
        """
        移除SQL构建器中的最后一个字符。
        """
        self.string_builder[-1] = self.string_builder[-1][:-1]

    def remove_last_keyword(self, keyword: str):
        """
        移除SQL构建器中的最后一个关键字。
        :param keyword: 要移除的关键字
        """
        if keyword:
            idx_last = "".join(self.string_builder).rfind(keyword)
            if idx_last > 0:
                self.string_builder = self.string_builder[:idx_last] + self.string_builder[idx_last + len(keyword):]

    def remove_last_blank(self):
        """
        移除SQL构建器中的最后一个空格。
        """
        if self.string_builder and self.string_builder[-1].endswith(" "):
            self.string_builder[-1] = self.string_builder[-1][:-1]

    def remove_first_type(self, first_type: OperatorType):
        """
        移除SQL构建器中第一个出现的类型。
        :param first_type: 要移除的类型
        """
        if first_type:
            idx_first = "".join(self.string_builder).find(first_type.code)
            if idx_first > 0:
                self.string_builder = self.string_builder[:idx_first] + self.string_builder[
                                                                        idx_first + len(first_type.code):]


from datetime import datetime
from typing import Any, Dict, List, Optional, Type, TypeVar

from mag_tools.bean.db_list import DbList
from mag_tools.bean.easy_map import EasyMap
from mag_tools.exception.dao_exception import DaoException
from mag_tools.utils.common.list_utils import ListUtils

from mag_db.bean.primary_key import PrimaryKey
from mag_db.bean.column import Column
from mag_db.bean.table import Table
from mag_db.bean.table_info import TableInfo
from mag_db.builder.delete import Delete
from mag_db.builder.insert import Insert
from mag_db.builder.select import Select
from mag_db.builder.update import Update
from mag_db.builder.where import Where
from mag_db.core.base_dao import BaseDao
from mag_db.enums.relation import Relation
from mag_db.sql.index_delete_executor import IndexDeleteExecutor
from mag_db.sql.index_insert_executor import IndexInsertExecutor
from mag_db.sql.index_query_executor import IndexQueryExecutor
from mag_db.sql.index_update_executor import IndexUpdateExecutor

T = TypeVar('T')
K = TypeVar('K')
V = TypeVar('V')

class SimpleDao(BaseDao):
    def __init__(self, table_names: List[str], bean_type: Type = None, column_names: List[str] = None,
                 primary_key_column: Optional[str] = None, distinct: bool = False, column_name_map: Optional[Dict[str, str]] = None,
                 auto_increment: bool = False):
        super().__init__()

        if not column_names and bean_type and hasattr(bean_type, '__annotations__'):
            column_names = list(bean_type.__annotations__.keys())

        tables = [Table(name) for name in table_names]
        columns = [Column(name) for name in column_names] if column_names else []
        primary_key = PrimaryKey([primary_key_column]) if primary_key_column else None

        self._table_info = TableInfo(tables, columns, primary_key, distinct=distinct, column_name_map=column_name_map, auto_increment=auto_increment)

    """
    常用的DAO操作类

      @version 1.0
      @Description 提供基本数据库操作, 如插入、更新等。
      注：SimpleDao类只提供简单的单表数据库操作，对复杂的SQL或多表操作可直接写SQL或继承BaseDao来完成。
      <p>
      数据源名称，其值必须与配置文件DBConfig.xml中的某个数据源名称一致。DBConfig.xml中可以定义多个数据源，
      但每个Dao基类只能对应其中一个数据源。数据源名称应当常量定义,以便事务处理使用。
      <p>
      Copyright: Copyright (c) 2015
      Company:
    """

    def insert(self, bean: T) -> int:
        """
         插入一条记录

         @param bean 要插入的数据
         @param <T>   列数值Bean类型
        """
        if isinstance(bean, list):
            raise DaoException("该方法只能插入单条数据")

        return self.insert_beans([bean])[0]

    def insert_map(self, field: EasyMap[K, V]) -> int:
        """
        插入操作
        """
        return self.insert_maps([field])[0]

    def insert_beans(self, beans: List[T]) -> List[int]:
        """
        多记录插入操作
        """
        return self.__do_insert_beans(beans)

    def insert_maps(self, fields: List[EasyMap[K, V]]) -> List[int]:
        """
        多记录插入操作
        """
        return self.__do_insert_maps(fields)

    def delete(self, where: Where) -> int:
        if where is None or where.is_empty:
            return 0

        delete_sql = Delete(self._table_info.table_names, where)
        delete = IndexDeleteExecutor(delete_sql.__str__(), self._session)
        if where.has_fields():
            delete.set_fields_of_where(where.fields)
        return delete.execute()

    def delete_by_id(self, id_: Any) -> int:
        if isinstance(id, list):
            raise DaoException("该方法只能删除单条数据")

        where = Where().column(self._table_info.default_primary_key, Relation.EQUAL, id_)
        return self.delete(where)

    def delete_by_ids(self, ids: List[Any]) -> int:
        where = Where().in_list(self._table_info.default_primary_key, ids)
        return self.delete(where)

    def clear(self) -> int:
        delete_sql = Delete(self._table_info.table_names, Where())
        delete = IndexDeleteExecutor(delete_sql.__str__(), self._session)
        return delete.execute()

    def update(self, param_bean: T, where: Where):
        update_sql = Update(self._table_info.table_names, self._table_info.get_column_names(), where)

        update = IndexUpdateExecutor(update_sql.__str__(), self._session)
        update.set_beans([param_bean], self._table_info.get_column_names(), self._table_info.column_name_map)
        if where.has_fields():
            update.set_fields_of_where(where.fields)

        return update.execute()

    def update_by_id(self, param_bean: T, id_: Any) -> int:
        where = Where().column(self._table_info.default_primary_key, Relation.EQUAL, id_)
        return self.update(param_bean, where)

    def update_by_id_of_bean(self, param_bean: T) -> int:
        id_ = getattr(param_bean, self._table_info.default_primary_key)
        if id_ is None:
            raise DaoException(f"主键[{self._table_info.default_primary_key}]不存在")

        return self.update_by_id(param_bean, id_)

    def update_beans(self, param_beans: List[T]) -> int:
        """
        根据主键更新多个Bean
        """
        num = 0
        for bean in param_beans:
            num += self.update_by_id_of_bean(bean)
        return num

    def update_map(self, field_map: Dict[K, V], where: Where) -> int:
        update_sql = Update(self._table_info.table_names, self._table_info.get_column_names(), where)
        update = IndexUpdateExecutor(update_sql.__str__(), self._session)
        update.set_maps([field_map], self._table_info.get_column_names())
        if where.has_fields():
            update.set_fields_of_where(where.fields)

        return update.execute()

    def update_map_by_id(self, field_map: Dict[K, V], id_: Any) -> int:
        where = Where().column(self._table_info.default_primary_key, Relation.EQUAL, id_)
        return self.update_map(field_map, where)

    def update_map_by_id_of_map(self, field_map: Dict[K, V]) -> int:
        id_ = field_map.get(self._table_info.default_primary_key, None)
        if id_ is None:
            raise DaoException(f"主键[{self._table_info.default_primary_key}]不存在")

        return self.update_map_by_id(field_map, id_)

    def update_maps(self, field_maps: List[Dict[K, V]]) -> int:
        num = 0
        for field_map in field_maps:
            num += self.update_map_by_id_of_map(field_map)
        return num

    def query(self, bean_class: type, where: Where) -> T:
        """
        查询数据库符合条件的指定记录
        """
        select = self.__get_select(where)

        query = IndexQueryExecutor(select.__str__(), self._session)
        if where.has_fields():
            query.set_fields_of_where(where.fields)

        return query.execute_bean(bean_class, self._table_info.get_column_names(True), self._table_info.column_name_map)

    def query_map(self, where: Where) -> Dict[str, T]:
        """
        查询数据库，查询结果保存在一个Map
        """
        select = self.__get_select(where)

        query = IndexQueryExecutor(select.__str__(), self._session)
        if where.has_fields():
            query.set_fields_of_where(where.fields)

        return query.execute_map(self._table_info.get_column_names(True))

    def query_by_id(self, bean_class: type, id_: str | int) -> T:
        """
        根据主键查询符合条件的指定记录
        """
        where = Where().column(self._table_info.default_primary_key, Relation.EQUAL, id_)
        return self.query(bean_class, where)

    def query_map_by_id(self, id_: str | int) -> Dict[str, Any]:
        """"
        根据主键查询
        """
        where = Where().column(self._table_info.default_primary_key, Relation.EQUAL, id_)
        return self.query_map(where)

    def exist_by_id(self, id_: str | int) ->bool:
        map_ = self.query_map_by_id(id_)
        return map_ is not None

    def exist_by_where(self, where: Where) ->bool:
        map_ = self.query_map(where)
        return map_ is not None

    def list(self, bean_class: type, where: Where) -> DbList[T]:
        """
        按条件列表数据库记录
        """
        select = self.__get_select(where)

        query = IndexQueryExecutor(select.__str__(), self._session)
        if where.has_fields():
            query.set_fields_of_where(where.fields)

        if where.is_page_query:
            return query.execute_beans_by_page(bean_class, self._table_info.get_column_names(True), self._table_info.column_name_map, where.get_page())
        else:
            return query.execute_beans(bean_class, self._table_info.get_column_names(True), self._table_info.column_name_map)

    def list_maps(self, where: Where) -> DbList[Dict[str, Any]]:
        select = self.__get_select(where)

        query = IndexQueryExecutor(select.__str__(), self._session)
        if where.has_fields():
            query.set_fields_of_where(where.fields)

        if where.is_page_query:
            return query.execute_maps_by_page(self._table_info.get_column_names(True), where.get_page())
        else:
            return query.execute_maps(self._table_info.get_column_names(True))

    def list_all(self, bean_class: type) -> 'DbList[T]':
        """
        列表数据库的全部记录
        :param bean_class: Bean类型
        """
        select = self.__get_select(Where())
        query = IndexQueryExecutor(select.__str__(), self._session)
        return query.execute_beans(bean_class, self._table_info.get_column_names(True), self._table_info.column_name_map)

    def list_all_maps(self) -> DbList[Dict[str, Any]]:
        select = self.__get_select(Where())
        query = IndexQueryExecutor(select.__str__(), self._session)
        return query.execute_maps(self._table_info.get_column_names(True))

    def get_latest_time(self, where: Where, order_by: str) -> Optional[datetime]:
        """
        查询表中最新记录的时间字段
        :param where: 查询条件
        :param order_by: 排序列名
        """
        where = where if where else Where()
        where.order(False, order_by).limit(1)

        record = self.query_map(where)
        return record.get(order_by, None) if record else None

    def __do_insert_beans(self, beans: List[T]) -> List[int]:
        """
        多记录插入操作
        当要插入的数据记录过多时，则分批插入
        @return 如有自增的主键，则返回主键值列表；否则返回null"""
        if not beans or len(beans) == 0:
            raise DaoException("至少要插入一条记录")

        blocks = ListUtils.split(beans, int(self._session.max_insert))

        insert_sql = Insert(self._table_info.table_names, self._table_info.get_column_names(), len(blocks[0]))
        insert = IndexInsertExecutor(insert_sql.__str__(), self._session)
        column_names_mapping = self._table_info.column_name_map

        insert_ids = []
        for i, block in enumerate(blocks):
            # 最后一块因记录数目不同，要重新生成insert
            if i == len(blocks) - 1:
                # insert_sql = Insert(self._table_info.table_names, self._table_info.get_column_names(), len(block))
                insert_sql = Insert(self._table_info.table_names, self._table_info.get_column_names(), 1)
                insert = IndexInsertExecutor(insert_sql.__str__(), self._session)

            insert_ids.extend(self.__insert_part_of_beans(insert, block, column_names_mapping))

        return insert_ids

    def __do_insert_maps(self, field_maps: List[EasyMap[K, V]]) -> List[int]:
        """
        多记录插入操作
        当要插入的数据记录过多时，则分批插入
        @return 如有自增的主键，则返回主键值列表；否则返回null
        """
        if not field_maps or len(field_maps) == 0:
            raise DaoException("至少要插入一条记录")

        blocks = ListUtils.split(field_maps, int(self._session.max_insert))
        insert_sql = Insert(self._table_info.table_names, self._table_info.get_column_names(), len(blocks[0]))
        insert = IndexInsertExecutor(insert_sql.__str__(), self._session)

        insert_ids = []
        for i, block in enumerate(blocks):
            if i == len(blocks) - 1:
                insert_sql = Insert(self._table_info.table_names, self._table_info.get_column_names(), len(block))
                insert = IndexInsertExecutor(insert_sql.__str__(), self._session)

            insert_ids.extend(self.__insert_part_of_maps(insert, block))

        return insert_ids

    def __insert_part_of_beans(self, insert: IndexInsertExecutor, beans: List[T], column_name_map: dict[str, str]) -> List[int]:
        insert.clear_parameters()
        insert.set_beans(beans, self._table_info.get_column_names(), column_name_map)

        return insert.execute()

    def __insert_part_of_maps(self, insert: IndexInsertExecutor, field_maps: List[EasyMap[K, V]]) -> List[int]:
        insert.clear_parameters()
        for field_map in field_maps:
            insert.set_maps([field_map], self._table_info.get_column_names())

        return insert.execute()

    def __get_select(self, where: Optional[Where]) -> Select:
        select = Select(self._table_info.table_names, self._table_info.get_column_names(True), where)
        if self._table_info.distinct:
            select.set_distinct()
        return select
